//*****************************************************************************
//
// Mystique.c
//
// This source file is organized into the following sections in the order 
// listed. Each section has further description on its organization.
//
// Includes
// Global data definitions
// Local defines and typedefs
// Local function references
// Local data definitions
// Global function definitions
// Local function definitions
//
//*****************************************************************************

// All includes files appear together at the beginning of this code. The order 
// of the includes begins with the most global in scope and continues with the 
// files of lesser scope.
//
// The first includes are those related to the C language and its library

#include <stdio.h>
#include <time.h>

// These includes are those referencing higher level subsystems and product 
// level files.

#include <windows.h>
#include "WinLib.h"

// This include is the interface definitions of this subsystem that 
// are used within this subsystem.

#include "resource.h"

//*****************************************************************************
//
// This section is reserved for global data definitions referenced through the 
// header file by higher subsystems and components of this subsystem. It is 
// best to use global data definitions as little as practical.
//
//*****************************************************************************

HINSTANCE hinstance; 
HWND hwndMain; 
char *manufacturer_name, *program_name, *main_window_title;

//*****************************************************************************
//
// This section is reserved for defines and typedefs used only in this file.
//
//*****************************************************************************

typedef struct
	{
	RECT xy;
	int mirror;
	} LINE;

//*****************************************************************************
//
// This section is reserved for local (static) function references used only 
// in this file. These references appear in the order that the functions are 
// defined after the global functions.
//
//*****************************************************************************

static LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam,
									LPARAM lParam );
static int Window_Create (HWND hWnd);
static void Window_Destroy (HWND hWnd);
static void Max_Screen (HWND hWnd, int toggle);
static DWORD WINAPI Draw_Thread (LPVOID lpParam);
static void random_delta (int *dx1, int *dy1, int *dx2, int *dy2);
static COLORREF random_color (void);
static void Line (HDC hDC, LINE *line);
static void XOR_Rectangle (HDC hDC, RECT *rect);
static int random (int low, int high);

static enum
	{
	DRAW_STATE_INIT,
	DRAW_STATE_RUN,
	DRAW_STATE_WAIT,
	DRAW_STATE_PAUSE,
	DRAW_STATE_STOP,
	} draw_state;
static enum
	{
	DRAW_VIEW_GO,
	DRAW_VIEW_PAUSE,
	DRAW_VIEW_STOP,
	} View_pause;
static unsigned View_speed = 0;
static unsigned View_color_cycle = 1;
static unsigned View_auto_pattern = 1;
static unsigned View_lines = 0;
static enum
	{
	VIEW_WIPE_BOX,
	VIEW_WIPE_NONE,
	VIEW_WIPE_UP,
	VIEW_WIPE_DOWN,
	VIEW_WIPE_LEFT,
	VIEW_WIPE_RIGHT,
	} View_wipe;
static HWND draw_hWnd;

//*****************************************************************************
//
// This section is reserved for local (static) data definitions used only 
// in this file.
//
//*****************************************************************************
static char source_file_name[] = __FILE__;
static int mirror;
static RECT client_rect;
static HMENU hmenu;

//*****************************************************************************
//
// This section is reserved for global function definitions referenced through 
// the header file by higher subsystems and components of this subsystem.
//
//*****************************************************************************

//*****************************************************************************
//
//*****************************************************************************
INT WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, INT nCmdShow)
	{
    WNDCLASS wc;
	ATOM atom;
	BOOL bRet;
	MSG msg;
	HACCEL haccel;
    UNREFERENCED_PARAMETER (hPrevInstance); 
    UNREFERENCED_PARAMETER (lpszCmdLine); 
	
	hinstance = hInstance;
	// Initialize global strings
	manufacturer_name = WinLib_LoadString (hinstance, IDS_MYSTIQUE_MANUFACTURER);
	program_name = WinLib_LoadString (hinstance, IDS_MYSTIQUE_NAME);
	main_window_title = WinLib_LoadString (hinstance, IDS_MYSTIQUE_TITLE);
	
    // Win32 will always set hPrevInstance to NULL, so lets check
    // things a little closer. This is because we only want a single
    // version of this app to run at a time
    hwndMain = FindWindow (program_name, NULL);
    if (hwndMain) 
		{
        // We found another version of ourself. Lets defer to it:
        if (IsIconic (hwndMain)) 
			{
            ShowWindow (hwndMain, SW_RESTORE);
			}
        SetForegroundWindow (hwndMain);
		
        // If this app actually had any command line functionality, we would
        // also want to communicate any action that our 'twin'
        // should now perform based on how the user tried to
        // execute us.
        return FALSE;
        }
	
	// Fill in window class structure with parameters that describe
	// the main window.
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = (WNDPROC) MainWndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hinstance;
	wc.hIcon         = LoadIcon (hinstance, program_name);
	wc.hCursor       = LoadCursor (NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW+1);
	wc.lpszMenuName =  "MAINMENU"; 
	wc.lpszClassName = program_name;
	   
	// Register the window class and return success/failure code.
	atom = RegisterClass (&wc);
	if (0 == atom)
		{
		WinLib_Error ("RegisterClass", 0);
		return FALSE;
		}
	
	// Create main window
	hwndMain = CreateWindow (program_name, main_window_title, WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hinstance, NULL);
	if (NULL == hwndMain)
		{
		WinLib_Error ("CreateWindow", 0);
		return FALSE;
		}

	hmenu = GetMenu (hwndMain);
	CheckMenuRadioItem (hmenu, ID_VIEW_GO, ID_VIEW_STOP, ID_VIEW_GO, MF_BYCOMMAND);
	CheckMenuItem (hmenu, ID_VIEW_MAXSCREEN, MF_UNCHECKED);
	CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_1, MF_BYCOMMAND);
	CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_VARIABLE, MF_BYCOMMAND);
	CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_BOX, MF_BYCOMMAND);
	
	ShowWindow (hwndMain, nCmdShow);
	
	haccel = LoadAccelerators(hInstance, program_name);

	// Main message loop:
	for (;;)
		{
		bRet = GetMessage (&msg, NULL, 0, 0);
		if (0 == bRet)
			break;
		if (-1 == bRet)
			{
			WinLib_Error ("GetMessage", 0);
			continue;
			}
		if (TranslateAccelerator (hwndMain, haccel, &msg))
			continue;
		TranslateMessage (&msg);
		DispatchMessage (&msg);
		}
	
	return (msg.wParam);
	}

//*****************************************************************************
//
// This section is reserved for local (static) function definitions used only 
// in this file.
//
//*****************************************************************************
static LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wParam,
									LPARAM lParam )
	{
	RECT rect;
	
	switch (msg) 
		{
		case WM_CREATE:
			return Window_Create (hWnd);
			
		case WM_ERASEBKGND:
			draw_state = DRAW_STATE_INIT;
			return 1;

		case WM_MOVE:
		case WM_SIZE:
			draw_state = DRAW_STATE_INIT;
			GetClientRect (hWnd, &rect);
			ClientToScreen(hWnd, (POINT *) &rect);
			MoveWindow(draw_hWnd, rect.left, rect.top, rect.right, rect.bottom, TRUE);
			break;

		case WM_SYSKEYDOWN:
			Max_Screen (hWnd, 0);
			break;

		case WM_COMMAND:
			switch (LOWORD(wParam))
				{
				case ID_FILE_EXIT:
					PostMessage (hWnd, WM_CLOSE, 0, 0);
					return 0;

				case ID_VIEW_INIT:
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_GO:
					View_pause = DRAW_VIEW_GO;
					CheckMenuRadioItem (hmenu, ID_VIEW_GO, ID_VIEW_STOP, ID_VIEW_GO, MF_BYCOMMAND);
					draw_state = DRAW_STATE_RUN;
					break;
				case ID_VIEW_PAUSE:
					View_pause = DRAW_VIEW_PAUSE;
					CheckMenuRadioItem (hmenu, ID_VIEW_GO, ID_VIEW_STOP, ID_VIEW_GO, MF_BYCOMMAND);
					draw_state = DRAW_STATE_RUN;
					break;
				case ID_VIEW_STOP:
					View_pause = DRAW_VIEW_STOP;
					CheckMenuRadioItem (hmenu, ID_VIEW_GO, ID_VIEW_STOP, ID_VIEW_STOP, MF_BYCOMMAND);
					draw_state = DRAW_STATE_RUN;
					break;
				case ID_VIEW_VARIABLE:
					View_lines = 0;
					CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_VARIABLE, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_ONE:
					View_lines = 1;
					mirror = 0;
					CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_ONE, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_TWO:
					View_lines = 2;
					mirror = 1;
					CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_TWO, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_FOUR:
					View_lines = 3;
					mirror = 2;
					CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_FOUR, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_EIGHT:
					View_lines = 4;
					mirror = 3;
					CheckMenuRadioItem (hmenu, ID_VIEW_VARIABLE, ID_VIEW_EIGHT, ID_VIEW_EIGHT, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_BOX:
					View_wipe = VIEW_WIPE_BOX;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_BOX, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_NONE:
					View_wipe = VIEW_WIPE_NONE;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_NONE, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_UP:
					View_wipe = VIEW_WIPE_UP;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_UP, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_DOWN:
					View_wipe = VIEW_WIPE_DOWN;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_DOWN, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_LEFT:
					View_wipe = VIEW_WIPE_LEFT;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_LEFT, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_RIGHT:
					View_wipe = VIEW_WIPE_RIGHT;
					CheckMenuRadioItem (hmenu, ID_VIEW_BOX, ID_VIEW_RIGHT, ID_VIEW_RIGHT, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_MAXSCREEN:
					Max_Screen (hWnd, 1);
					break;
				case ID_VIEW_AUTO:
					View_auto_pattern = 1 - View_auto_pattern;
					CheckMenuItem (hmenu, ID_VIEW_AUTO, View_auto_pattern ? MF_CHECKED : MF_UNCHECKED);
					break;
				case ID_VIEW_COLOR:
					View_color_cycle = 1 - View_color_cycle;
					CheckMenuItem (hmenu, ID_VIEW_COLOR, View_color_cycle ? MF_CHECKED : MF_UNCHECKED);
					break;
				case ID_VIEW_1:
					View_speed = 0;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_1, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_2:
					View_speed = 1;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_2, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_3:
					View_speed = 3;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_3, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_4:
					View_speed = 7;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_4, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_5:
					View_speed = 15;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_5, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_6:
					View_speed = 31;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_6, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_7:
					View_speed = 63;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_7, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_8:
					View_speed = 127;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_8, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				case ID_VIEW_9:
					View_speed = 255;
					CheckMenuRadioItem (hmenu, ID_VIEW_1, ID_VIEW_9, ID_VIEW_9, MF_BYCOMMAND);
					draw_state = DRAW_STATE_INIT;
					break;
				}
			break;
			
		case WM_DESTROY:
			draw_state = DRAW_STATE_STOP;
			Sleep (10);
			Window_Destroy (hWnd);
			PostQuitMessage (0);
			return 0;
		}
	return (DefWindowProc (hWnd, msg, wParam, lParam));
	}

//*****************************************************************************
//
// Initialization required to create the window. 
//
// return success (0) or failure (-1) code
//
//*****************************************************************************
static int Window_Create (HWND hWnd)
	{
    DWORD dwThreadId; 
    HANDLE hThread;
	RECT rect;

	GetClientRect (hWnd, &rect);
	ClientToScreen(hWnd, (POINT *) &rect);

	// Create draw window
	draw_hWnd = CreateWindow ("STATIC", "", WS_POPUP | WS_VISIBLE,
		rect.left, rect.top, rect.right, rect.bottom,
		hWnd, NULL, hinstance, NULL);
	if (NULL == draw_hWnd)
		{
		WinLib_Error ("CreateWindow", 0);
		return FALSE;
		}

	hThread = CreateThread (NULL, 0, Draw_Thread, draw_hWnd, 0, &dwThreadId);
	SetThreadPriority (hThread, THREAD_PRIORITY_IDLE);
    CloseHandle (hThread);

	return 0;
	}

//*****************************************************************************
//
// release resources used to create the window. 
//
//*****************************************************************************
static void Window_Destroy (HWND hWnd)
	{
    UNREFERENCED_PARAMETER (hWnd); 
	
	}

//*****************************************************************************
//
// Max Screen off or toggle. 
//
//*****************************************************************************
static void Max_Screen (HWND hWnd, int toggle)
	{
	static int state;

	if (!toggle && !state)
		return;

	if (state)
		{
		ShowWindow (draw_hWnd, SW_RESTORE);
		CheckMenuItem (hmenu, ID_VIEW_MAXSCREEN, MF_UNCHECKED);
		}
	else
		{
		ShowWindow (draw_hWnd, SW_MAXIMIZE);
		CheckMenuItem (hmenu, ID_VIEW_MAXSCREEN, MF_CHECKED);
		}
	SetFocus (hWnd);
	state = 1- state;
	draw_state = DRAW_STATE_INIT;
	}

//*****************************************************************************
//
// Drawing thread state routine
//
//*****************************************************************************
static DWORD WINAPI Draw_Thread (LPVOID lpParam)
	{
	HWND hWnd;
	HDC hDC;
	HBRUSH hbrush, hbrush_old;
	HPEN hpen1, hpen2, hpen_old, hpen_rect;
	RECT rect;
	LINE lines[1024];
	unsigned i, max_i;
	int x1, y1, x2, y2;
	int dx1, dy1, dx2, dy2;

	hWnd = (HWND) lpParam;
	hDC = GetDC (hWnd);
	SetROP2(hDC, R2_XORPEN);
	srand ((unsigned) time (NULL));
	random_delta (&dx1, &dy1, &dx2, &dy2);
	hpen1 = NULL;
	hpen2 = NULL;
	hpen_rect = NULL;
	hpen_old = SelectObject (hDC, hpen1);
	hbrush_old = SelectObject (hDC, GetStockObject(NULL_BRUSH));
	random_delta (&dx1, &dy1, &dx2, &dy2);

	while (DRAW_STATE_INIT == draw_state)
		{
		GetClientRect (hWnd, &client_rect);
		x1 = client_rect.left;
		y1 = client_rect.top;
		x2 = client_rect.right;
		y2 = client_rect.bottom;
		if (hpen1 == hpen2)
			hpen2 = NULL;
		if (hpen1)
			{
			DeleteObject (hpen1);
			hpen1 = NULL;
			}
		hbrush = CreateSolidBrush (random_color());
		FillRect (hDC, &client_rect, hbrush);
		DeleteObject (hbrush);
		SetRectEmpty (&rect);
		Sleep (100);
		draw_state = DRAW_STATE_RUN;
		max_i = min (SIZEOF (lines), ((unsigned) client_rect.right));
		for (;;)
			{

			if (DRAW_STATE_RUN != draw_state)
				break;

			if (NULL == hpen2)
				hpen2 = CreatePen(PS_SOLID, 0, random_color ());
			for (i = 0; i < max_i; ++i)
				{
				if (DRAW_VIEW_STOP == View_pause)
					draw_state = DRAW_STATE_PAUSE;

				while (DRAW_STATE_PAUSE == draw_state)
					Sleep (100);

				if (DRAW_STATE_RUN != draw_state)
					break;

				/* Draw the wipe.
				 */
				if (IsRectEmpty (&rect))
					{
					CopyRect (&rect, &client_rect);
					hpen_rect = CreatePen(PS_SOLID, 0, random_color ());
					}
				SelectObject (hDC, hpen_rect);
				XOR_Rectangle (hDC, &rect);

				/* Draw the lines.
				 */
				if (hpen1)
					{
					if (0 == (i & View_speed))
						Sleep (1);
					SelectObject (hDC, hpen1);
					Line (hDC, &lines[i]);
					}
				lines[i].xy.left = x1;
				lines[i].xy.top = y1;
				lines[i].xy.right = x2;
				lines[i].xy.bottom = y2;
				lines[i].mirror = mirror;
				SelectObject (hDC, hpen2);
				Line (hDC, &lines[i]);
				x1 += dx1;
				if ((x1 < client_rect.left) || (x1 > client_rect.right))
					{
					dx1 = -dx1;
					x1 += dx1;
					if (View_auto_pattern)
						random_delta (&dx1, &dy1, &dx2, &dy2);
					}
				x2 += dx2;
				if ((x2 < client_rect.left) || (x2 > client_rect.right))
					{
					dx2 = -dx2;
					x2 += dx2;
					if (View_auto_pattern)
						random_delta (&dx1, &dy1, &dx2, &dy2);
					}
				y1 += dy1;
				if ((y1 < client_rect.top) || (y1 > client_rect.bottom))
					{
					dy1 = -dy1;
					y1 += dy1;
					if (View_auto_pattern)
						random_delta (&dx1, &dy1, &dx2, &dy2);
					}
				y2 += dy2;
				if ((y2 < client_rect.top) || (y2 > client_rect.bottom))
					{
					dy2 = -dy2;
					y2 += dy2;
					if (View_auto_pattern)
						random_delta (&dx1, &dy1, &dx2, &dy2);
					}
				}
			if (View_color_cycle)
				{
				if (hpen1)
					DeleteObject (hpen1);
				hpen1 = hpen2;
				hpen2 = NULL;
				}
			else
				{
				if (hpen1 != hpen2)
					{
					if (hpen1)
						DeleteObject (hpen1);
					hpen1 = hpen2;
					}
				}
			}
		}
	SelectObject (hDC, hbrush_old);
	SelectObject (hDC, hpen_old);
	if (hpen1)
		DeleteObject (hpen1);
	if (hpen2)
		DeleteObject (hpen2);
	ReleaseDC(hWnd, hDC);
	return 0;
	}

//*****************************************************************************
//
// get a useful random delta pattern. Don't allow too little movement.
//
//*****************************************************************************
static void random_delta (int *dx1, int *dy1, int *dx2, int *dy2)
	{
	static int counter = 0;

	if (DRAW_VIEW_PAUSE == View_pause)
		draw_state = DRAW_STATE_PAUSE;

	if (0 < --counter)
		return;

	if (View_lines)
		mirror = View_lines - 1;
	else
		mirror = random (0, 3);
	
	do
		{
		*dx1 = random (-3, 3);
		*dx2 = random (-3, 3);
		*dy1 = random (-3, 3);
		*dy2 = random (-3, 3);
		counter = 4 * abs(*dx1) + abs(*dx2) + abs(*dy1) + abs(*dy2);
		}
	while (0 == counter);
	}

//*****************************************************************************
//
// get a random color. Don't use too dark a color.
//
//*****************************************************************************
static COLORREF random_color (void)
	{
	unsigned total;
	union
		{
		RGBQUAD rgbquad;
		COLORREF colorref;
		}color;

	do
		{
		total = color.rgbquad.rgbRed = (unsigned char) random (0, 255);
		total += color.rgbquad.rgbGreen = (unsigned char) random (0, 255);
		total += color.rgbquad.rgbBlue = (unsigned char) random (0, 255);
		}
	while (128 > total);
	color.rgbquad.rgbReserved = 0;
	return color.colorref;
	}

//*****************************************************************************
//
// draw a line between two values with optional mirrors.
//
//*****************************************************************************
static void Line (HDC hDC, LINE *line)
	{
	RECT r;

	CopyRect (&r, &line->xy);
	switch (line->mirror)
		{
		case 3:
			MoveToEx (hDC, r.right, r.bottom, NULL);
			LineTo (hDC, r.left, client_rect.bottom - r.top);
			MoveToEx (hDC, r.right, client_rect.bottom - r.bottom, NULL);
			LineTo (hDC, client_rect.right - r.left, r.top);
			MoveToEx (hDC, client_rect.right - r.right, r.bottom, NULL);
			LineTo (hDC, client_rect.right - r.left, client_rect.bottom - r.top);
			MoveToEx (hDC, client_rect.right - r.right, client_rect.bottom - r.bottom, NULL);
			LineTo (hDC, r.left, r.top);
		case 2:
			MoveToEx (hDC, r.left, client_rect.bottom - r.top, NULL);
			LineTo (hDC, r.right, client_rect.bottom - r.bottom);
			MoveToEx (hDC, client_rect.right - r.left, r.top, NULL);
			LineTo (hDC, client_rect.right - r.right, r.bottom);
		case 1:
			MoveToEx (hDC, client_rect.right - r.left, client_rect.bottom - r.top, NULL);
			LineTo (hDC, client_rect.right - r.right, client_rect.bottom - r.bottom);
		case 0:
		default:
			MoveToEx (hDC, r.left, r.top, NULL);
			LineTo (hDC, r.right, r.bottom);
		}
	}

//*****************************************************************************
//
// draw a rectangle that disappears with xor and shrink.
//
//*****************************************************************************
static void XOR_Rectangle (HDC hDC, RECT *rect)
	{

	switch (View_wipe)
		{
		case VIEW_WIPE_NONE:
			break;
		case VIEW_WIPE_BOX:
			rect->right--;
			rect->bottom--;
			MoveToEx (hDC, rect->left, rect->top, NULL);
			LineTo (hDC, rect->right, rect->top);
			LineTo (hDC, rect->right, rect->bottom);
			if (!IsRectEmpty (rect))
				{
				LineTo (hDC, rect->left, rect->bottom);
				LineTo (hDC, rect->left, rect->top);
				}
			rect->left++;
			rect->top++;
			break;
		case VIEW_WIPE_UP:
			rect->bottom--;
			MoveToEx (hDC, rect->left, rect->bottom, NULL);
			LineTo (hDC, rect->right, rect->bottom);
			break;
		case VIEW_WIPE_DOWN:
			MoveToEx (hDC, rect->left, rect->top, NULL);
			LineTo (hDC, rect->right, rect->top);
			rect->top++;
			break;
			break;
		case VIEW_WIPE_LEFT:
			MoveToEx (hDC, rect->left, rect->top, NULL);
			LineTo (hDC, rect->left, rect->bottom);
			rect->left++;
			break;
			break;
		case VIEW_WIPE_RIGHT:
			rect->right--;
			MoveToEx (hDC, rect->right, rect->top, NULL);
			LineTo (hDC, rect->right, rect->bottom);
			break;
			break;
		}
	}

//*****************************************************************************
//
// get a random number somewhere between two values.
//
//*****************************************************************************
static int random (int low, int high)
	{

	return low + rand () % (1 + high - low);
	}